/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// r_surf.c: surface-related refresh code

#include "quakedef.h"
#include "r_local.h"

drawsurf_t	r_drawsurf;

int				lightleft, sourcesstep, blocksize, sourcetstep;
int				lightdelta, lightdeltastep;
int				lightright, lightleftstep, lightrightstep, blockdivshift;
unsigned		blockdivmask;
void			*prowdestbase;
unsigned char	*pbasesource;
int				surfrowbytes;	// used by ASM files
unsigned		*r_lightptr;
int				r_stepback;
int				r_lightwidth;
int				r_numhblocks, r_numvblocks;
unsigned char	*r_source, *r_sourcemax;

void R_DrawSurfaceBlock8_mip0 (void);
void R_DrawSurfaceBlock8_mip1 (void);
void R_DrawSurfaceBlock8_mip2 (void);
void R_DrawSurfaceBlock8_mip3 (void);

static void	(*surfmiptable[4])(void) = {
	R_DrawSurfaceBlock8_mip0,
	R_DrawSurfaceBlock8_mip1,
	R_DrawSurfaceBlock8_mip2,
	R_DrawSurfaceBlock8_mip3
};



unsigned		blocklights[18*18];

/*
===============
R_AddDynamicLights
===============
*/
void R_AddDynamicLights (void)
{
	msurface_t *surf;
	int			lnum;
	int			sd, td;
	float		dist, rad, minlight;
	vec3_t		impact, local;
	vec3_t		origin_for_ent; // mankrip - dynamic lights on moving brush models fix
	int			s, t;
	int			smax, tmax;
	mtexinfo_t	*tex;

	surf = r_drawsurf.surf;
	smax = (surf->extents[0]>>4)+1;
	tmax = (surf->extents[1]>>4)+1;
	tex = surf->texinfo;

	for (lnum=0 ; lnum<MAX_DLIGHTS ; lnum++)
	{
		if ( !(surf->dlightbits & (1<<lnum) ) )
			continue;		// not lit by this light

		rad = cl_dlights[lnum].radius;
		VectorSubtract (cl_dlights[lnum].origin, currententity->origin, origin_for_ent); // mankrip - dynamic lights on moving brush models fix
		dist = DotProduct (origin_for_ent, surf->plane->normal) - // mankrip - dynamic lights on moving brush models fix - edited
				surf->plane->dist;
		rad -= fabs(dist);
		minlight = cl_dlights[lnum].minlight;
		if (rad < minlight)
			continue;
		minlight = rad - minlight;

		// mankrip - dynamic lights on moving brush models fix - begin
		impact[0] = origin_for_ent[0] - surf->plane->normal[0]*dist;
		impact[1] = origin_for_ent[1] - surf->plane->normal[1]*dist;
		impact[2] = origin_for_ent[2] - surf->plane->normal[2]*dist;
		// mankrip - dynamic lights on moving brush models fix - end

		local[0] = DotProduct (impact, tex->vecs[0]) + tex->vecs[0][3];
		local[1] = DotProduct (impact, tex->vecs[1]) + tex->vecs[1][3];

		local[0] -= surf->texturemins[0];
		local[1] -= surf->texturemins[1];

		for (t = 0 ; t<tmax ; t++)
		{
			td = local[1] - t*16;
			if (td < 0)
				td = -td;
			for (s=0 ; s<smax ; s++)
			{
				sd = local[0] - s*16;
				if (sd < 0)
					sd = -sd;

				if (sd > td)
					dist = sd + (td>>1);
				else
					dist = td + (sd>>1);
				if (dist < minlight)
				{
				//	#ifdef QUAKE2
					if (cl_dlights[lnum].dark)
					{
						unsigned temp = (rad - dist)*256; // mankrip
						int i = t*smax + s;
						if (blocklights[i] > temp)
							blocklights[i] -= temp;
						else
							blocklights[i] = 0;
					}
					else
				//	#endif
						blocklights[t*smax + s] += (rad - dist)*256;
				}
			}
		}
	}
}

/*
===============
R_BuildLightMap

Combine and scale multiple lightmaps into the 8.8 format in blocklights
===============
*/
void R_BuildLightMap (void)
{
	int			smax, tmax;
	int			t;
	int			i, size;
	byte		*lightmap;
	unsigned	scale;
	int			maps;
	int			count, remainder; // mankrip
	unsigned	*blocks; // mankrip
	msurface_t	*surf;

	surf = r_drawsurf.surf;

	smax = (surf->extents[0]>>4)+1;
	tmax = (surf->extents[1]>>4)+1;
	size = smax*tmax; // in most cases, 64*64

	if (r_fullbright.value || !cl.worldmodel->lightdata)
	{
		for (i=0 ; i<size ; i++)
			blocklights[i] = 31<<8;//0; // mankrip - fullbright BSP textures fix
		return;
	}

// clear to ambient
	// mankrip - begin
	#if 1
	memset (blocklights, r_refdef.ambientlight<<8, sizeof (unsigned) * size);
	#else
	// mankrip - end
	for (i=0 ; i<size ; i++)
		blocklights[i] = r_refdef.ambientlight<<8;
	#endif // mankrip

	lightmap = surf->samples;
	count = size / 32; // mankrip
	remainder = size % 32; // mankrip

// add all the lightmaps
	if (lightmap)
		for (maps = 0 ; maps < MAXLIGHTMAPS && surf->styles[maps] != 255 ; maps++)
		{
		//	scale = r_drawsurf.lightadj[maps];	// 8.8 fraction
		//	if (!r_externbsp_lit.value || currententity != cl_entities && currententity->model->name[0] != '*') // mankrip
				scale = (unsigned) ( (float)r_drawsurf.lightadj[maps] * currententity->lightlevel); // mankrip
			#if 1 // mankrip
			for (i=0 ; i<size ; i++)
				blocklights[i] += lightmap[i] * scale;
			#endif // mankrip
			lightmap += size;	// skip to next lightmap
		}

// add all the dynamic lights
	if (surf->dlightframe == r_framecount)
		R_AddDynamicLights ();

// bound, invert, and shift
	// mankrip - begin
	#if 1
	blocks = blocklights;
	for (i = 0 ; i < count ; i++, blocks += 32)
	{
		t = (255*256 - (int)blocks[31]) >> (8 - VID_CBITS); blocks[31] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[30]) >> (8 - VID_CBITS); blocks[30] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[29]) >> (8 - VID_CBITS); blocks[29] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[28]) >> (8 - VID_CBITS); blocks[28] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[27]) >> (8 - VID_CBITS); blocks[27] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[26]) >> (8 - VID_CBITS); blocks[26] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[25]) >> (8 - VID_CBITS); blocks[25] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[24]) >> (8 - VID_CBITS); blocks[24] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[23]) >> (8 - VID_CBITS); blocks[23] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[22]) >> (8 - VID_CBITS); blocks[22] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[21]) >> (8 - VID_CBITS); blocks[21] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[20]) >> (8 - VID_CBITS); blocks[20] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[19]) >> (8 - VID_CBITS); blocks[19] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[18]) >> (8 - VID_CBITS); blocks[18] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[17]) >> (8 - VID_CBITS); blocks[17] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[16]) >> (8 - VID_CBITS); blocks[16] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[15]) >> (8 - VID_CBITS); blocks[15] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[14]) >> (8 - VID_CBITS); blocks[14] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[13]) >> (8 - VID_CBITS); blocks[13] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[12]) >> (8 - VID_CBITS); blocks[12] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[11]) >> (8 - VID_CBITS); blocks[11] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[10]) >> (8 - VID_CBITS); blocks[10] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[ 9]) >> (8 - VID_CBITS); blocks[ 9] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[ 8]) >> (8 - VID_CBITS); blocks[ 8] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[ 7]) >> (8 - VID_CBITS); blocks[ 7] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[ 6]) >> (8 - VID_CBITS); blocks[ 6] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[ 5]) >> (8 - VID_CBITS); blocks[ 5] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[ 4]) >> (8 - VID_CBITS); blocks[ 4] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[ 3]) >> (8 - VID_CBITS); blocks[ 3] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[ 2]) >> (8 - VID_CBITS); blocks[ 2] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[ 1]) >> (8 - VID_CBITS); blocks[ 1] = (t < (1 << 6)) ? (1 << 6) : t;
		t = (255*256 - (int)blocks[ 0]) >> (8 - VID_CBITS); blocks[ 0] = (t < (1 << 6)) ? (1 << 6) : t;
	}
	switch (remainder)
	{
		case 31: t = (255*256 - (int)blocks[31]) >> (8 - VID_CBITS); blocks[31] = (t < (1 << 6)) ? (1 << 6) : t;
		case 30: t = (255*256 - (int)blocks[30]) >> (8 - VID_CBITS); blocks[30] = (t < (1 << 6)) ? (1 << 6) : t;
		case 29: t = (255*256 - (int)blocks[29]) >> (8 - VID_CBITS); blocks[29] = (t < (1 << 6)) ? (1 << 6) : t;
		case 28: t = (255*256 - (int)blocks[28]) >> (8 - VID_CBITS); blocks[28] = (t < (1 << 6)) ? (1 << 6) : t;
		case 27: t = (255*256 - (int)blocks[27]) >> (8 - VID_CBITS); blocks[27] = (t < (1 << 6)) ? (1 << 6) : t;
		case 26: t = (255*256 - (int)blocks[26]) >> (8 - VID_CBITS); blocks[26] = (t < (1 << 6)) ? (1 << 6) : t;
		case 25: t = (255*256 - (int)blocks[25]) >> (8 - VID_CBITS); blocks[25] = (t < (1 << 6)) ? (1 << 6) : t;
		case 24: t = (255*256 - (int)blocks[24]) >> (8 - VID_CBITS); blocks[24] = (t < (1 << 6)) ? (1 << 6) : t;
		case 23: t = (255*256 - (int)blocks[23]) >> (8 - VID_CBITS); blocks[23] = (t < (1 << 6)) ? (1 << 6) : t;
		case 22: t = (255*256 - (int)blocks[22]) >> (8 - VID_CBITS); blocks[22] = (t < (1 << 6)) ? (1 << 6) : t;
		case 21: t = (255*256 - (int)blocks[21]) >> (8 - VID_CBITS); blocks[21] = (t < (1 << 6)) ? (1 << 6) : t;
		case 20: t = (255*256 - (int)blocks[20]) >> (8 - VID_CBITS); blocks[20] = (t < (1 << 6)) ? (1 << 6) : t;
		case 19: t = (255*256 - (int)blocks[19]) >> (8 - VID_CBITS); blocks[19] = (t < (1 << 6)) ? (1 << 6) : t;
		case 18: t = (255*256 - (int)blocks[18]) >> (8 - VID_CBITS); blocks[18] = (t < (1 << 6)) ? (1 << 6) : t;
		case 17: t = (255*256 - (int)blocks[17]) >> (8 - VID_CBITS); blocks[17] = (t < (1 << 6)) ? (1 << 6) : t;
		case 16: t = (255*256 - (int)blocks[16]) >> (8 - VID_CBITS); blocks[16] = (t < (1 << 6)) ? (1 << 6) : t;
		case 15: t = (255*256 - (int)blocks[15]) >> (8 - VID_CBITS); blocks[15] = (t < (1 << 6)) ? (1 << 6) : t;
		case 14: t = (255*256 - (int)blocks[14]) >> (8 - VID_CBITS); blocks[14] = (t < (1 << 6)) ? (1 << 6) : t;
		case 13: t = (255*256 - (int)blocks[13]) >> (8 - VID_CBITS); blocks[13] = (t < (1 << 6)) ? (1 << 6) : t;
		case 12: t = (255*256 - (int)blocks[12]) >> (8 - VID_CBITS); blocks[12] = (t < (1 << 6)) ? (1 << 6) : t;
		case 11: t = (255*256 - (int)blocks[11]) >> (8 - VID_CBITS); blocks[11] = (t < (1 << 6)) ? (1 << 6) : t;
		case 10: t = (255*256 - (int)blocks[10]) >> (8 - VID_CBITS); blocks[10] = (t < (1 << 6)) ? (1 << 6) : t;
		case  9: t = (255*256 - (int)blocks[ 9]) >> (8 - VID_CBITS); blocks[ 9] = (t < (1 << 6)) ? (1 << 6) : t;
		case  8: t = (255*256 - (int)blocks[ 8]) >> (8 - VID_CBITS); blocks[ 8] = (t < (1 << 6)) ? (1 << 6) : t;
		case  7: t = (255*256 - (int)blocks[ 7]) >> (8 - VID_CBITS); blocks[ 7] = (t < (1 << 6)) ? (1 << 6) : t;
		case  6: t = (255*256 - (int)blocks[ 6]) >> (8 - VID_CBITS); blocks[ 6] = (t < (1 << 6)) ? (1 << 6) : t;
		case  5: t = (255*256 - (int)blocks[ 5]) >> (8 - VID_CBITS); blocks[ 5] = (t < (1 << 6)) ? (1 << 6) : t;
		case  4: t = (255*256 - (int)blocks[ 4]) >> (8 - VID_CBITS); blocks[ 4] = (t < (1 << 6)) ? (1 << 6) : t;
		case  3: t = (255*256 - (int)blocks[ 3]) >> (8 - VID_CBITS); blocks[ 3] = (t < (1 << 6)) ? (1 << 6) : t;
		case  2: t = (255*256 - (int)blocks[ 2]) >> (8 - VID_CBITS); blocks[ 2] = (t < (1 << 6)) ? (1 << 6) : t;
		case  1: t = (255*256 - (int)blocks[ 1]) >> (8 - VID_CBITS); blocks[ 1] = (t < (1 << 6)) ? (1 << 6) : t;
		case  0: t = (255*256 - (int)blocks[ 0]) >> (8 - VID_CBITS); blocks[ 0] = (t < (1 << 6)) ? (1 << 6) : t;
		default: break;
	}
	#else
	// mankrip - end
	for (i=0 ; i<size ; i++)
	{
		t = (255*256 - (int)blocklights[i]) >> (8 - VID_CBITS);

		blocklights[i] = (t < (1 << 6)) ? (1 << 6) : t; // mankrip
	}
	#endif
}


/*
===============
R_TextureAnimation

Returns the proper texture for a given time and base texture
===============
*/
texture_t *R_TextureAnimation (texture_t *base)
{
	if (currententity->frame)
	{
		if (base->alternate_anims)
			base = base->alternate_anims;
	}

	// mankrip - edited - begin
	if (base->anim_total)
	{
		int
			reletive = (int)(cl.time*10) % base->anim_total
		,	count = 0
			;
	// mankrip - edited - end

		while (base->anim_min > reletive || base->anim_max <= reletive)
		{
			base = base->anim_next;
			if (!base)
				Sys_Error ("R_TextureAnimation: broken cycle");
			if (++count > 100)
				Sys_Error ("R_TextureAnimation: infinite cycle");
		}
	} // mankrip

	return base;
}


/*
===============
R_DrawSurface
===============
*/
void R_DrawSurface (void)
{
	unsigned char	*basetptr;
	int				smax, tmax, twidth;
	int				u;
	int				soffset, basetoffset, texwidth;
	int				horzblockstep;
	unsigned char	*pcolumndest;
	void			(*pblockdrawer)(void);
	texture_t		*mt;

// calculate the lightings
	R_BuildLightMap ();

	surfrowbytes = r_drawsurf.rowbytes;

	mt = r_drawsurf.texture;

	r_source = (byte *)mt + mt->offsets[r_drawsurf.surfmip];

// the fractional light values should range from 0 to (VID_GRADES - 1) << 16
// from a source range of 0 - 255

	texwidth = mt->width >> r_drawsurf.surfmip;

	blocksize = 16 >> r_drawsurf.surfmip;
	blockdivshift = 4 - r_drawsurf.surfmip;
	blockdivmask = (1 << blockdivshift) - 1;

	r_lightwidth = (r_drawsurf.surf->extents[0]>>4)+1;

	r_numhblocks = r_drawsurf.surfwidth >> blockdivshift;
	r_numvblocks = r_drawsurf.surfheight >> blockdivshift;

//==============================

	if (r_pixbytes == 1)
	{
		pblockdrawer = surfmiptable[r_drawsurf.surfmip];
	// TODO: only needs to be set when there is a display settings change
		horzblockstep = blocksize;
	}
	else
	{
		pblockdrawer = R_DrawSurfaceBlock16;
	// TODO: only needs to be set when there is a display settings change
		horzblockstep = blocksize << 1;
	}

	smax = mt->width >> r_drawsurf.surfmip;
	twidth = texwidth;
	tmax = mt->height >> r_drawsurf.surfmip;
	sourcetstep = texwidth;
	r_stepback = tmax * twidth;

	r_sourcemax = r_source + (tmax * smax);

	soffset = r_drawsurf.surf->texturemins[0];
	basetoffset = r_drawsurf.surf->texturemins[1];

// << 16 components are to guarantee positive values for %
	soffset = ((soffset >> r_drawsurf.surfmip) + (smax << 16)) % smax;
	basetptr = &r_source[((((basetoffset >> r_drawsurf.surfmip)
		+ (tmax << 16)) % tmax) * twidth)];

	pcolumndest = r_drawsurf.surfdat;

	for (u=0 ; u<r_numhblocks; u++)
	{
		r_lightptr = blocklights + u;

		prowdestbase = pcolumndest;

		pbasesource = basetptr + soffset;

		(*pblockdrawer)();

		soffset = soffset + blocksize;
		if (soffset >= smax)
			soffset = 0;

		pcolumndest += horzblockstep;
	}
}


//=============================================================================

#if	!id386

// mankrip - begin
static int
	v
,	i
,	lightstep
,	light
	;
static byte
	* psource
,	* prowdest
	;
// mankrip - end
/*
================
R_DrawSurfaceBlock8_mip0
================
*/
void R_DrawSurfaceBlock8_mip0 (void)
{
	psource = pbasesource;
	prowdest = prowdestbase;

	for (v=0 ; v<r_numvblocks ; v++)
	{
	// FIXME: make these locals?
	// FIXME: use delta rather than both right and left, like ASM?
		lightleft = r_lightptr[0];
		lightright = r_lightptr[1];
		r_lightptr += r_lightwidth;
		lightleftstep = (r_lightptr[0] - lightleft) >> 4;
		lightrightstep = (r_lightptr[1] - lightright) >> 4;

		for (i=0 ; i<16 ; i++)
		{
			lightstep = (lightleft - lightright) >> 4; // mankrip

			light = lightright;

			// mankrip - begin
			prowdest[15] = vid.colormap[ (light & 0xFF00) + psource[15]]; light += lightstep;
			prowdest[14] = vid.colormap[ (light & 0xFF00) + psource[14]]; light += lightstep;
			prowdest[13] = vid.colormap[ (light & 0xFF00) + psource[13]]; light += lightstep;
			prowdest[12] = vid.colormap[ (light & 0xFF00) + psource[12]]; light += lightstep;
			prowdest[11] = vid.colormap[ (light & 0xFF00) + psource[11]]; light += lightstep;
			prowdest[10] = vid.colormap[ (light & 0xFF00) + psource[10]]; light += lightstep;
			prowdest[ 9] = vid.colormap[ (light & 0xFF00) + psource[ 9]]; light += lightstep;
			prowdest[ 8] = vid.colormap[ (light & 0xFF00) + psource[ 8]]; light += lightstep;
			prowdest[ 7] = vid.colormap[ (light & 0xFF00) + psource[ 7]]; light += lightstep;
			prowdest[ 6] = vid.colormap[ (light & 0xFF00) + psource[ 6]]; light += lightstep;
			prowdest[ 5] = vid.colormap[ (light & 0xFF00) + psource[ 5]]; light += lightstep;
			prowdest[ 4] = vid.colormap[ (light & 0xFF00) + psource[ 4]]; light += lightstep;
			prowdest[ 3] = vid.colormap[ (light & 0xFF00) + psource[ 3]]; light += lightstep;
			prowdest[ 2] = vid.colormap[ (light & 0xFF00) + psource[ 2]]; light += lightstep;
			prowdest[ 1] = vid.colormap[ (light & 0xFF00) + psource[ 1]]; light += lightstep;
			prowdest[ 0] = vid.colormap[ (light & 0xFF00) + psource[ 0]]; light += lightstep;
			// mankrip - end

			psource += sourcetstep;
			lightright += lightrightstep;
			lightleft += lightleftstep;
			prowdest += surfrowbytes;
		}

		if (psource >= r_sourcemax)
			psource -= r_stepback;
	}
}


/*
================
R_DrawSurfaceBlock8_mip1
================
*/
void R_DrawSurfaceBlock8_mip1 (void)
{
	psource = pbasesource;
	prowdest = prowdestbase;

	for (v=0 ; v<r_numvblocks ; v++)
	{
	// FIXME: make these locals?
	// FIXME: use delta rather than both right and left, like ASM?
		lightleft = r_lightptr[0];
		lightright = r_lightptr[1];
		r_lightptr += r_lightwidth;
		lightleftstep = (r_lightptr[0] - lightleft) >> 3;
		lightrightstep = (r_lightptr[1] - lightright) >> 3;

		for (i=0 ; i<8 ; i++)
		{
			lightstep = (lightleft - lightright) >> 3; // mankrip

			light = lightright;

			// mankrip - begin
			prowdest[ 7] = vid.colormap[ (light & 0xFF00) + psource[ 7]]; light += lightstep;
			prowdest[ 6] = vid.colormap[ (light & 0xFF00) + psource[ 6]]; light += lightstep;
			prowdest[ 5] = vid.colormap[ (light & 0xFF00) + psource[ 5]]; light += lightstep;
			prowdest[ 4] = vid.colormap[ (light & 0xFF00) + psource[ 4]]; light += lightstep;
			prowdest[ 3] = vid.colormap[ (light & 0xFF00) + psource[ 3]]; light += lightstep;
			prowdest[ 2] = vid.colormap[ (light & 0xFF00) + psource[ 2]]; light += lightstep;
			prowdest[ 1] = vid.colormap[ (light & 0xFF00) + psource[ 1]]; light += lightstep;
			prowdest[ 0] = vid.colormap[ (light & 0xFF00) + psource[ 0]]; light += lightstep;
			// mankrip - end

			psource += sourcetstep;
			lightright += lightrightstep;
			lightleft += lightleftstep;
			prowdest += surfrowbytes;
		}

		if (psource >= r_sourcemax)
			psource -= r_stepback;
	}
}


/*
================
R_DrawSurfaceBlock8_mip2
================
*/
void R_DrawSurfaceBlock8_mip2 (void)
{
	psource = pbasesource;
	prowdest = prowdestbase;

	for (v=0 ; v<r_numvblocks ; v++)
	{
	// FIXME: make these locals?
	// FIXME: use delta rather than both right and left, like ASM?
		lightleft = r_lightptr[0];
		lightright = r_lightptr[1];
		r_lightptr += r_lightwidth;
		lightleftstep = (r_lightptr[0] - lightleft) >> 2;
		lightrightstep = (r_lightptr[1] - lightright) >> 2;

		for (i=0 ; i<4 ; i++)
		{
			lightstep = (lightleft - lightright) >> 2; // mankrip

			light = lightright;

			// mankrip - begin
			prowdest[ 3] = vid.colormap[ (light & 0xFF00) + psource[ 3]]; light += lightstep;
			prowdest[ 2] = vid.colormap[ (light & 0xFF00) + psource[ 2]]; light += lightstep;
			prowdest[ 1] = vid.colormap[ (light & 0xFF00) + psource[ 1]]; light += lightstep;
			prowdest[ 0] = vid.colormap[ (light & 0xFF00) + psource[ 0]]; light += lightstep;
			// mankrip - end

			psource += sourcetstep;
			lightright += lightrightstep;
			lightleft += lightleftstep;
			prowdest += surfrowbytes;
		}

		if (psource >= r_sourcemax)
			psource -= r_stepback;
	}
}


/*
================
R_DrawSurfaceBlock8_mip3
================
*/
void R_DrawSurfaceBlock8_mip3 (void)
{
	psource = pbasesource;
	prowdest = prowdestbase;

	for (v=0 ; v<r_numvblocks ; v++)
	{
	// FIXME: make these locals?
	// FIXME: use delta rather than both right and left, like ASM?
		lightleft = r_lightptr[0];
		lightright = r_lightptr[1];
		r_lightptr += r_lightwidth;
		lightleftstep = (r_lightptr[0] - lightleft) >> 1;
		lightrightstep = (r_lightptr[1] - lightright) >> 1;

		lightstep = (lightleft - lightright) >> 1; // mankrip
		light = lightright;
		// mankrip - begin
		prowdest[ 1] = vid.colormap[ (light & 0xFF00) + psource[ 1]]; light += lightstep;
		prowdest[ 0] = vid.colormap[ (light & 0xFF00) + psource[ 0]]; light += lightstep;
		// mankrip - end
		psource += sourcetstep;
		lightright += lightrightstep;
		lightleft += lightleftstep;
		prowdest += surfrowbytes;

		lightstep = (lightleft - lightright) >> 1; // mankrip
		light = lightright;
		// mankrip - begin
		prowdest[ 1] = vid.colormap[ (light & 0xFF00) + psource[ 1]]; light += lightstep;
		prowdest[ 0] = vid.colormap[ (light & 0xFF00) + psource[ 0]]; light += lightstep;
		// mankrip - end
		psource += sourcetstep;
		prowdest += surfrowbytes;

		if (psource >= r_sourcemax)
			psource -= r_stepback;
	}
}


/*
================
R_DrawSurfaceBlock16

FIXME: make this work
================
*/
void R_DrawSurfaceBlock16 (void)
{
#if 0
	int				k;
	unsigned char	*psource;
	int				lighttemp, lightstep, light;
	unsigned short	*prowdest;

	prowdest = (unsigned short *)prowdestbase;

	for (k=0 ; k<blocksize ; k++)
	{
		unsigned short	*pdest;
		unsigned char	pix;
		int				b;

		psource = pbasesource;
		lighttemp = lightright - lightleft;
		lightstep = lighttemp >> blockdivshift;

		light = lightleft;
		pdest = prowdest;

		for (b=0; b<blocksize; b++)
		{
			pix = *psource;
			*pdest = vid.colormap16[(light & 0xFF00) + pix];
			psource += sourcesstep;
			pdest++;
			light += lightstep;
		}

		pbasesource += sourcetstep;
		lightright += lightrightstep;
		lightleft += lightleftstep;
		prowdest = (unsigned short *)((long)prowdest + surfrowbytes);
	}

	prowdestbase = prowdest;
#endif
}

#endif


//============================================================================

/*
================
R_GenTurbTile
================
*/
/* // mankrip - removed - begin
void R_GenTurbTile (pixel_t *pbasetex, void *pdest)
{
	int		*turb;
	int		i, j, s, t;
	byte	*pd;

	turb = sintable + ((int)(cl.time*SPEED)&(CYCLE-1));
	pd = (byte *)pdest;

	for (i=0 ; i<TILE_SIZE ; i++)
	{
		for (j=0 ; j<TILE_SIZE ; j++)
		{
			s = (((j << 16) + turb[i & (CYCLE-1)]) >> 16) & 63;
			t = (((i << 16) + turb[j & (CYCLE-1)]) >> 16) & 63;
			*pd++ = *(pbasetex + (t<<6) + s);
		}
	}
}
*/ // mankrip - removed - end


/*
================
R_GenTurbTile16
================
*/
/* // mankrip - removed - begin
void R_GenTurbTile16 (pixel_t *pbasetex, void *pdest)
{
	int				*turb;
	int				i, j, s, t;
	unsigned short	*pd;

	turb = sintable + ((int)(cl.time*SPEED)&(CYCLE-1));
	pd = (unsigned short *)pdest;

	for (i=0 ; i<TILE_SIZE ; i++)
	{
		for (j=0 ; j<TILE_SIZE ; j++)
		{
			s = (((j << 16) + turb[i & (CYCLE-1)]) >> 16) & 63;
			t = (((i << 16) + turb[j & (CYCLE-1)]) >> 16) & 63;
			*pd++ = d_8to16table[*(pbasetex + (t<<6) + s)];
		}
	}
}
*/ // mankrip - removed - end


/*
================
R_GenTile
================
*/
/* // mankrip - removed - begin
void R_GenTile (msurface_t *psurf, void *pdest)
{
	if (psurf->flags & SURF_DRAWTURB)
	{
		if (r_pixbytes == 1)
		{
			R_GenTurbTile ((pixel_t *)
				((byte *)psurf->texinfo->texture + psurf->texinfo->texture->offsets[0]), pdest);
		}
		else
		{
			R_GenTurbTile16 ((pixel_t *)
				((byte *)psurf->texinfo->texture + psurf->texinfo->texture->offsets[0]), pdest);
		}
	}
	else if (psurf->flags & SURF_DRAWSKY)
	{
		if (r_pixbytes == 1)
		{
			R_GenSkyTile (pdest);
		}
		else
		{
			R_GenSkyTile16 (pdest);
		}
	}
	else
	{
		Sys_Error ("Unknown tile type");
	}
}
*/ // mankrip - removed - end

